﻿-- constants
USE_PAD	= false -- shkim - gamePad is not supported since now

TWO_PI = 2.0 * 3.14159
DEG_TO_RAD = (3.14159 / 180.0)
RAD_TO_DEG = (180.0 / 3.14159)

X_AXIS = hkVector4.new( 1, 0, 0 )
Y_AXIS = hkVector4.new( 0, 1, 0 )
Z_AXIS = hkVector4.new( 0, 0, 1 )

UP_DIRECTION = hkVector4.new( 0, 0, 1 )
FORWARD_DIRECTION = hkVector4.new( 0, -1, 0 )
LEFT_DIRECTION = hkVector4.new( 1, 0, 0 )
RIGHT_DIRECTION = hkVector4.new( -1, 0, 0 )

DESIRED_DIRECTION = FORWARD_DIRECTION -- test

DEF_ACCEL = 5.0
MAX_VELOCITY = 10
INIT_VELOCITY = 3.0

-- interpolates a scalar value
function interpolate( a, b, t )
	return ((1.0 - t) * a) + (t * b)		
end

-- returns a value clamped between min and max
function clamp( value, min, max )
    if( value > max ) then return max end
    if( value < min ) then return min end
    return value
end

-- clamps an angle to the rangle of [-2PI, 2PI]
function angleDiff( diff )
    if (diff > TWO_PI/2) then diff = diff - TWO_PI end
    if (diff < -TWO_PI/2) then diff = diff + TWO_PI end
    return diff
end


function makeRadBy360(angle)
	-- map the angle into the range 0 to 2 pi
	if ( angle < 0 ) then
		angle = angle + TWO_PI 
    end
	
	angle = angle - TWO_PI * math.floor( angle / TWO_PI )
		
	return angle;
end


-- the lua state of the character
g_characterState = {
	
	m_timestep = 0.0,
	m_timeElapsed = 0.0,
	
	m_angle = 0.0,
	
	--m_aimAngleX = 0.0,
	--m_aimAngleY = 0.0,
	
	--m_forward = hkVector4.new(),
		
	--m_aimTargetCameraAngleX = 0.0,
	--m_aimTargetCameraAngleY = 0.0,
	
	m_characterWidth = 0.01,
	
	m_currentPos = hkVector4.new(),
	m_lastPos = hkVector4.new(),
	m_lastAngle = 0.0, -- pad angle 
	m_moveSpeed = 0.0,
	
	m_attackPressed = 0,
}

	-- root level update.  called everyframe to update the character.
function onUpdateRoot()

	-- record the timestep
	g_characterState.m_timestep = hkbGetTimestep()
	g_characterState.m_timeElapsed = g_characterState.m_timeElapsed + g_characterState.m_timestep
	
	--g_gamePad:update( g_characterState.m_timestep )
	
	-- Compute the character angle (cached for efficiency)
	local fwd = hkVector4.new( 1, 0, 0 )
	fwd:setRotatedDir( hkbGetOldWorldFromModel():getRotation(), X_AXIS)
	g_characterState.m_angle = math.atan2( fwd[1], fwd[0] )
	
	local globalAngle = hkbGetVariable("GlobalAngle") -- direction to oppenent player
	if( globalAngle == nil ) then globalAngle = 0.0 end;
	
	g_characterState.m_angle = g_characterState.m_angle - globalAngle
	
	-- test code
	--local curDirRad = hkbGetVariable("Direction")*DEG_TO_RAD
	--if( curDirRad ~= nil ) then
		--local curDirVec = hkVector4.new(math.cos(curDirRad), math.sin(curDirRad), 0)
		--local desiredDir = curDirVec:dot3(DESIRED_DIRECTION)
		--if(desiredDir > 0) then
		--	hkbSetVariable("MoveTypeID", 1)
		--else
		--	hkbSetVariable("MoveTypeID", 0)
		--end	
	--end
	
	-- store off the characters forward direction
	--g_characterState.m_forward:setRotatedDir(hkbGetOldWorldFromModel():getRotation(), FORWARD_DIRECTION)
	
	local moveDist = g_characterState.m_currentPos - g_characterState.m_lastPos
	moveDist[2] = 0.0
	local velocity = moveDist:length3()

	g_characterState.m_moveSpeed = 0.5*(g_characterState.m_moveSpeed+velocity)
	--hkbPrint(g_characterState.m_moveSpeed)
		
	g_characterState.m_lastPos = g_characterState.m_currentPos
	g_characterState.m_currentPos = hkbGetOldWorldFromModel():getTranslation()
		
end

