
CAMERA_MODE_DEFAULT = 0
CAMERA_MODE_STRAFE = 1
CAMERA_MODE_CLIMBING_LEDGE = 2

g_camera = {
	m_from = hkVector4.new( 0.0, 4.0, 2.5 ),
	m_to = hkVector4.new( 0.0, 0.0, 1.75 ),
	m_up = hkVector4.new( 0, 0, 1 ),
	m_forward = hkVector4.new( 0, 0, 0),
	
	m_angle = 0,

	m_mode = 0,
			
	m_targetLeadAmount = hkVector4.new(),	
	m_cameraFollowDistance = 5.0,
	
	m_orbitMaxSpeedX = 1.75,
	m_orbitMaxSpeedY = 1,	
	m_orbitSpeedX = 0.0,
	m_orbitSpeedY = 0.0,
	
	m_cameraArmLocalSpace = hkVector4.new(0, 1.0, 0.0)
}
 
	-- updates the camera
function g_camera:update( timestep )
	
	-- set the camera mode based on the active state
	self.m_mode = CAMERA_MODE_DEFAULT

	-- compute the camera angle (cached for efficiency)
	self.m_forward[0] = self.m_from[0] - self.m_to[0]
	self.m_forward[1] = self.m_from[1] - self.m_to[1]	
	self.m_forward[2] = 0
	self.m_forward:normalize3()
	self.m_angle = math.atan2( self.m_forward[0], self.m_forward[1] )			
	
	-- update the orbit speed
	--if( g_gamePad:isRightStickInDeadZone() ) then					 		
	--	self.m_orbitSpeedX = self.m_orbitSpeedX - self.m_orbitSpeedX * 0.1
	--	self.m_orbitSpeedY = self.m_orbitSpeedY - self.m_orbitSpeedY * 0.1
	--else
	--	self.m_orbitSpeedX = clamp(self.m_orbitSpeedX + g_gamePad.m_rightStickX * 0.05, -self.m_orbitMaxSpeedX, self.m_orbitMaxSpeedX)
	--	self.m_orbitSpeedY = clamp(self.m_orbitSpeedY + g_gamePad.m_rightStickY * 0.05, -self.m_orbitMaxSpeedY, self.m_orbitMaxSpeedY)
	--end
	
	-- update the camera logic
	g_camera:update3dPersonCamera( timestep )
	
	-- Set the HBT variables which control it
	hkbSetVariable("inputcamerafrom", self.m_from)
	hkbSetVariable("inputcamerato", self.m_to)
	hkbSetVariable("inputcameraup", self.m_up)	
end

function g_camera:update3dPersonCamera( timestep )	
		
	local checkVisible = true

	-- compute the target of the camera.  the target is the character's location
	-- plus a height offset (changes based on camera mode) plus a fraction of
	-- their velocity.
	local targetTo = nil
	
	-- add height offset based on mode
	if( self.m_mode == CAMERA_MODE_DEFAULT ) then
		targetTo = hkbGetOldWorldFromModel():getTranslation()
		targetTo[2] = targetTo[2] + 1.6
	end
	
	-- add a fraction of the character's velocity so that camera "leads" the character
	local scaledVelocity = hkbGetCharacterControllerVelocity()
	local targetToNoLead = hkVector4.new(targetTo)
	scaledVelocity:mul4(10 * timestep)		
	self.m_targetLeadAmount:setInterpolate4(self.m_targetLeadAmount, scaledVelocity, 5.0 * timestep )	
	targetTo:add4(self.m_targetLeadAmount)
	
	-- change the camera follow distance based on mode
	local cameraFollowDistance = nil
	if( self.m_mode == CAMERA_MODE_DEFAULT ) then
		cameraFollowDistance = g_normalFollowDistance
	end	
	self.m_cameraFollowDistance = interpolate( self.m_cameraFollowDistance, cameraFollowDistance, 0.05)
	
	-- compute the look at location of the camera.  this is at the same height as the
 	-- camera's target but rotated by the camera's arm
	local targetFrom = hkVector4.new()
	targetFrom = hkbGetOldWorldFromModel():getTranslation()
	targetFrom[2] = targetTo[2]
	targetFrom:addMul4(self.m_cameraArmLocalSpace, self.m_cameraFollowDistance)	
	
	if (checkVisible) then
		-- make sure the camera doesn't go through the wall
		local cameraDirection = hkVector4.new()
		cameraDirection:setSub4(targetFrom, targetTo)
		cameraDirection:normalize3()
			
		local hitTestTo = targetToNoLead
		local hitTestFrom = targetFrom
		--hitTestTo:setAddMul4(targetTo, cameraDirection, 0.3)
			
		local hit, hitFraction, hitNormal, hitPosition = hkbCastRay(targetToNoLead, hitTestFrom, 3)		
			
		if( hitFraction > 0.1 ) then
								 
			targetFrom:setAddMul4(hitPosition, cameraDirection, -0.05)

		end 
	end
	-- smoothly interpolate to the new targets
	self.m_to:setInterpolate4(self.m_to, targetTo, .25)
	self.m_from:setInterpolate4(self.m_from, targetFrom, .33)
	self.m_up = UP_DIRECTION
	
end

